package com.ekexiu.project.bridge.servlet;

import org.jfw.util.context.JfwAppContext;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;
import org.jfw.util.json.JsonService;

import static java.lang.Thread.sleep;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * Created by TT on 2019/3/6.
 */
public class EnvDataServlet extends HttpServlet {
    private static final long serialVersionUID = 749116393196185727L;
    private DataSource dataSource;
    private List<EnvThread> envThreadList = new CopyOnWriteArrayList<EnvThread>();
    private ListenThread listenThread;

    private AtomicBoolean running = new AtomicBoolean(false);

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        res.setDateHeader("Expires", 0);
        res.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
        res.addHeader("Cache-Control", "post-check=0, pre-check=0");
        res.setHeader("Pragma", "no-cache");
        List<Map<String, Object>> result = null;
        if (null == req.getCharacterEncoding()) {
            req.setCharacterEncoding("UTF-8");
            res.setCharacterEncoding("UTF-8");
        }
        res.setContentType("text/html");
        PrintWriter out = res.getWriter();
        try {
            String param = req.getParameter("bt");
            String bt = "00000000000000";
            if (null != param && param.length() != 0) {
                bt = param;
            }
            param = req.getParameter("et");
            String et = "99999999999999";
            if (null != param && param.length() != 0) {
                et = param;
            }
            Connection con = this.dataSource.getConnection();
            try {
                result = query(con, bt, et);
            } finally {
                try {
                    con.close();
                } catch (Throwable _m_1) {
                }
            }
        } catch (Exception e) {
            JsonService.write(e, out);
            return;
        }
        out.write("<!DOCTYPE html><html><head><meta http-equiv='Content-Type' content='text/html; charset=UTF-8'></head><body>");
        out.write("<table border=\"1\">\n" +
                "<tr>\n" +
                "  <td>id</td>\n" +
                "  <td>显示内容</td>\n" +
                "  <td>单位</td>\n" +
                "  <td>值</td>\n" +
                "  <td>更新时间</td>\n" +
                "</tr>");
        for (Map<String, Object> map : result) {
            out.write("<tr>\n" +
                    "  <td>" + map.get("item_id") + "</td>\n" +
                    "  <td>" + map.get("caption") + "</td>\n" +
                    "  <td>" + map.get("unit") + "</td>\n" +
                    "  <td>" + map.get("item_value") + "</td>\n" +
                    "  <td>" + map.get("item_time") + "</td>\n" +
                    "</tr>");
        }
        out.write("</table></body></html>");
    }

    public List<Map<String, Object>> query(Connection con, final String bt, final String et) throws SQLException {
        return JdbcUtil.queryMaps(con, "SELECT e.item_id,COALESCE(d.caption, '没有对应值') as caption,COALESCE(d.unit, '没有对应值') as unit,e.item_value,e.item_time FROM env_data e LEFT JOIN env_id_dict d on e.item_id = d.item_id where e.item_time>= ? and e.item_time<=?", new PreparedStatementConfig() {
            @Override
            public void config(PreparedStatement preparedStatement) throws SQLException {
                preparedStatement.setString(1, bt);
                preparedStatement.setString(2, et);
            }
        });
    }

    @Override
    public void init(ServletConfig config) throws ServletException {
        dataSource = JfwAppContext.getDataSource();
        running.set(true);
        try {
            ServerSocket serverSocket = new ServerSocket(4000);
            listenThread = new ListenThread(serverSocket);
            listenThread.start();
        } catch (IOException e) {
            e.printStackTrace();
        }


    }

    @Override
    public void destroy() {
        this.running.set(false);
        if (listenThread != null) {
            listenThread.interrupt();

            for (; ; ) {
                if (this.envThreadList.size() > 0) {
                    for (Thread thread : this.envThreadList) {
                        thread.interrupt();
                    }
                    try {
                        sleep(500);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                } else {
                    break;
                }
            }

        }
    }

    class ListenThread extends Thread {
        private ServerSocket serverSocket;


        public ListenThread(ServerSocket serverSocket) {
            this.serverSocket = serverSocket;
        }

        @Override
        public void run() {
            while (running.get()) {
                try {
                    Socket socket = serverSocket.accept();
                    EnvThread envThread = new EnvThread(socket);
                    envThreadList.add(envThread);
                    envThread.start();
                } catch (Exception e) {
                }
            }
        }
    }

    class EnvThread extends Thread {
        private Socket socket;
        private char timeArray[] = new char[14];

        public EnvThread(Socket socket) {
            this.socket = socket;
        }

        @Override
        public void run() {
            int idx, len, ret, offset;
            while (running.get()) {
                try {
                    InputStream inputStream = socket.getInputStream();
                    byte[] head = new byte[52];
                    idx = 0;
                    len = 52;
                    while (len > 0) {
                        ret = inputStream.read(head, idx, len);
                        if (ret >= 0) {
                            if(ret > 0){
                                System.out.print("read head:");
                                headtoString(head,idx,ret);
                            }
                            idx += ret;
                            len -= ret;
                        } else {
                            throw new IOException("a");
                        }
                    }
                    len = head[51] & 0xFF;
                    len <<= 8;
                    len += head[50] & 0xFF;
                    len <<= 8;
                    len += head[49] & 0xFF;
                    len <<= 8;
                    len += head[48] & 0xFF;


                    byte[] body = new byte[len];
                    idx = 0;
                    while (len > 0) {
                        ret = inputStream.read(body, idx, len);
                        if (ret >= 0) {
                            if(ret>0){
                                    System.out.print("read body:");
                                    headtoString(body,idx,ret);

                            }
                            idx += ret;
                            len -= ret;
                        } else {
                            throw new IOException("a");
                        }
                    }
                    len = body.length;
                    offset = 0;
                    List<EnvData> list = new ArrayList<EnvData>();
                    while (len >= offset) {
                        int id = body[offset + 3] & 0xFF;
                        id <<= 8;
                        id += body[offset + 2] & 0xFF;
                        id <<= 8;
                        id += body[offset + 1] & 0xFF;
                        id <<= 8;
                        id += body[offset] & 0xFF;

                        double value = bytes2Double(body, offset + 4);

                        timeArray[0] = (char) body[offset + 13];
                        timeArray[1] = (char) body[offset + 14];
                        timeArray[2] = (char) body[offset + 15];
                        timeArray[3] = (char) body[offset + 16];
                        timeArray[4] = (char) body[offset + 18];
                        timeArray[5] = (char) body[offset + 19];
                        timeArray[6] = (char) body[offset + 21];
                        timeArray[7] = (char) body[offset + 22];
                        timeArray[8] = (char) body[offset + 24];
                        timeArray[9] = (char) body[offset + 25];
                        timeArray[10] = (char) body[offset + 27];
                        timeArray[11] = (char) body[offset + 28];
                        timeArray[12] = (char) body[offset + 30];
                        timeArray[13] = (char) body[offset + 31];
                        offset += 32;
                        len -= 32;
                        EnvData envData = new EnvData();
                        envData.setItemId(id);
                        envData.setItemValue(value);
                        envData.setItemTime(new String(timeArray));
                        list.add(envData);
                    }

                    try {
                        Connection con = dataSource.getConnection();
                        try {
                            for (EnvData envData : list) {
                                PreparedStatement ps = con.prepareStatement("INSERT INTO ENV_DATA (ITEM_ID,ITEM_VALUE,ITEM_TIME) VALUES (?,?,?)");
                                ps.setInt(1, envData.getItemId());
                                ps.setDouble(2, envData.getItemValue());
                                ps.setString(3, envData.getItemTime());
                                ps.executeUpdate();
                            }
                            con.commit();
                        } catch (SQLException e) {
                            con.rollback();
                        } finally {
                            con.close();
                        }
                    } catch (Throwable thr) {

                    }


                } catch (Exception e) {
                    break;
                }
            }
            try {
                socket.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            envThreadList.remove(this);
        }

    }

    public static void headtoString(byte[] head,int idx,int len) {
        int rlen = idx +len;
        System.out.print("[");
        for(int i = idx; i < rlen;++i){
            System.out.print(head[i]);
            System.out.print(",");
        }
        System.out.println("]");
    }

    public static double bytes2Double(byte[] arr, int index) {
        long value = 0;
        for (int i = 0; i < 8; i++) {
            value |= ((long) (arr[index + i] & 0xff)) << (8 * i);
        }
        return Double.longBitsToDouble(value);
    }

    public static class EnvData {
        private int itemId;
        private double itemValue;
        private String itemTime;

        public int getItemId() {
            return itemId;
        }

        public void setItemId(int itemId) {
            this.itemId = itemId;
        }

        public double getItemValue() {
            return itemValue;
        }

        public void setItemValue(double itemValue) {
            this.itemValue = itemValue;
        }

        public String getItemTime() {
            return itemTime;
        }

        public void setItemTime(String itemTime) {
            this.itemTime = itemTime;
        }
    }


}

